CompilerIf #PB_Compiler_OS = #PB_OS_Windows
  #LIBUSB_PATH_MAX = 512
  
  Macro StrAlign 
  EndMacro
  
CompilerElseIf #PB_Compiler_OS = #PB_OS_Linux
  #LIBUSB_PATH_MAX = 4096+1
  
  Macro StrAlign 
    Align #PB_Structure_AlignC
  EndMacro
  
CompilerEndIf

; Device And/Or Interface Class codes

#USB_CLASS_PER_INTERFACE =   0  ; For DeviceClass
#USB_CLASS_AUDIO         =   1
#USB_CLASS_COMM          =   2
#USB_CLASS_HID           =   3
#USB_CLASS_PRINTER       =   7
#USB_CLASS_MASS_STORAGE  =   8
#USB_CLASS_HUB           =   9
#USB_CLASS_DATA          =  10
#USB_CLASS_VENDOR_SPEC   = $ff

; Descriptor types
#USB_DT_DEVICE    = $01
#USB_DT_CONFIG    = $02
#USB_DT_STRING    = $03
#USB_DT_INTERFACE = $04
#USB_DT_ENDPOINT  = $05

#USB_DT_HID      = $21
#USB_DT_REPORT   = $22
#USB_DT_PHYSICAL = $23
#USB_DT_HUB      = $29

; Descriptor sizes per descriptor type
#USB_DT_DEVICE_SIZE         =  18
#USB_DT_CONFIG_SIZE         =   9
#USB_DT_INTERFACE_SIZE      =   9
#USB_DT_ENDPOINT_SIZE       =   7
#USB_DT_ENDPOINT_AUDIO_SIZE =   9  ; Audio extension
#USB_DT_HUB_NONVAR_SIZE     =   7


; All standard descriptors have these 2 fields in common
Structure usb_descriptor_header StrAlign
  bLength.a
  bDescriptorType.a
EndStructure

; String descriptor
Structure usb_string_descriptor StrAlign
  bLength.a
  bDescriptorType.a
  wData.u[1]
EndStructure

; HID descriptor
Structure usb_hid_descriptor StrAlign
  bLength.a
  bDescriptorType.a
  bcdHID.u
  bCountryCode.a
  bNumDescriptors.a
EndStructure

; Endpoint descriptor
#USB_MAXENDPOINTS  = 32
Structure usb_endpoint_descriptor StrAlign
  bLength.a
  bDescriptorType.a
  bEndpointAddress.a
  bmAttributes.a
  wMaxPacketSize.u
  bInterval.a
  bRefresh.a
  bSynchAddress.a

  *extra ; Extra descriptors
  extralen.i
EndStructure

#USB_ENDPOINT_ADDRESS_MASK     = $0f    ; in bEndpointAddress
#USB_ENDPOINT_DIR_MASK         = $80

#USB_ENDPOINT_TYPE_MASK        = $03    ; in bmAttributes
#USB_ENDPOINT_TYPE_CONTROL     =   0
#USB_ENDPOINT_TYPE_ISOCHRONOUS =   1
#USB_ENDPOINT_TYPE_BULK        =   2
#USB_ENDPOINT_TYPE_INTERRUPT   =   3

; Interface descriptor
#USB_MAXINTERFACES = 32
Structure usb_interface_descriptor StrAlign
  bLength.a
  bDescriptorType.a
  bInterfaceNumber.a
  bAlternateSetting.a
  bNumEndpoints.a
  bInterfaceClass.a
  bInterfaceSubClass.a
  bInterfaceProtocol.a
  iInterface.a

  *endpoint.usb_endpoint_descriptor

  *extra ; Extra descriptors
  extralen.i
EndStructure

#USB_MAXALTSETTING = 128  ; Hard limit

Structure usb_interface StrAlign
  *altsetting.usb_interface_descriptor

  num_altsetting.i
EndStructure

; Configuration descriptor information..
#USB_MAXCONFIG = 8
Structure usb_config_descriptor StrAlign
  bLength.a
  bDescriptorType.a
  wTotalLength.u
  bNumInterfaces.a
  bConfigurationValue.a
  iConfiguration.a
  bmAttributes.a
  MaxPower.a

  *Interface.usb_interface

  *extra  ; Extra descriptors
  extralen.i
EndStructure

; Device descriptor
Structure usb_device_descriptor StrAlign
  bLength.a
  bDescriptorType.a
  bcdUSB.u
  bDeviceClass.a
  bDeviceSubClass.a
  bDeviceProtocol.a
  bMaxPacketSize0.a
  idVendor.u
  idProduct.u
  bcdDevice.u
  iManufacturer.a
  iProduct.a
  iSerialNumber.a
  bNumConfigurations.a
EndStructure

Structure usb_ctrl_setup StrAlign
  bRequestType.a
  bRequest.a
  wValue.u
  wIndex.u
  wLength.u
EndStructure

; Standard requests

#USB_REQ_GET_STATUS         = $00
#USB_REQ_CLEAR_FEATURE      = $01
; $02 is reserved
#USB_REQ_SET_FEATURE        = $03
; $04 is reserved
#USB_REQ_SET_ADDRESS        = $05
#USB_REQ_GET_DESCRIPTOR     = $06
#USB_REQ_SET_DESCRIPTOR     = $07
#USB_REQ_GET_CONFIGURATION  = $08
#USB_REQ_SET_CONFIGURATION  = $09
#USB_REQ_GET_INTERFACE      = $0A
#USB_REQ_SET_INTERFACE      = $0B
#USB_REQ_SYNCH_FRAME        = $0C

#USB_TYPE_STANDARD          = ($00 << 5)
#USB_TYPE_CLASS             = ($01 << 5)
#USB_TYPE_VENDOR            = ($02 << 5)
#USB_TYPE_RESERVED          = ($03 << 5)

#USB_RECIP_DEVICE           = $00
#USB_RECIP_INTERFACE        = $01
#USB_RECIP_ENDPOINT         = $02
#USB_RECIP_OTHER            = $03

; Various libusb API related stuff

#USB_ENDPOINT_IN    = $80
#USB_ENDPOINT_OUT   = $00

; Error codes
#USB_ERROR_BEGIN    = 500000

; This is supposed To look weird. This file is generated from autoconf
; And I didn't want to make this too complicated.

; #USB_LE16_TO_CPU(x)

; Device reset types For usb_reset_ex.
; http://msdn.microsoft.com/en-us/library/ff537269%28VS.85%29.aspx
; http://msdn.microsoft.com/en-us/library/ff537243%28v=vs.85%29.aspx

#USB_RESET_TYPE_RESET_PORT = (1 << 0)
#USB_RESET_TYPE_CYCLE_PORT = (1 << 1)
#USB_RESET_TYPE_FULL_RESET = (#USB_RESET_TYPE_CYCLE_PORT | #USB_RESET_TYPE_RESET_PORT)


; Data types
; struct usb_device
; struct usb_bus

Structure usb_device StrAlign
  *next.usb_device
  *prev.usb_device

  filename.a[#LIBUSB_PATH_MAX]

  *bus.usb_bus

  descriptor.usb_device_descriptor
  *config.usb_config_descriptor

  *dev  ; Darwin support

  devnum.a

  num_children.a
  *children.usb_device
EndStructure

Structure usb_bus StrAlign
  *next.usb_bus
  *prev.usb_bus

  dirname.a[#LIBUSB_PATH_MAX]

  *devices.usb_device
  location.l

  *root_dev.usb_device
EndStructure

; Version information, Windows specific
Structure dll StrAlign
  major.i
  minor.i
  micro.i
  nano.i
EndStructure

Structure driver StrAlign
  major.i
  minor.i
  micro.i
  nano.i
EndStructure
 
Structure usb_version StrAlign
  dll.dll
  driver.driver
EndStructure

Structure usb_dev_handle StrAlign
  fd.l

  *bus.usb_bus
  *device.usb_device

  config.l
  hinterface.l
  altsetting.l

  ; Added by RMT so implementations can store other per-open-device Data
  *impl_info
EndStructure


CompilerSelect #PB_Compiler_OS
    
  CompilerCase #PB_OS_Windows
    
    CompilerIf #PB_Compiler_Processor = #PB_Processor_x86
      #LibUsbFile = "libusb.lib"
    CompilerElse
      #LibUsbFile = "libusb_x64.lib"
    CompilerEndIf
    
  CompilerCase #PB_OS_Linux
    #LibUsbFile = "-lusb"
    
CompilerEndSelect

ImportC #LibUsbFile
  usb_get_version()
  usb_init()
  usb_find_busses()
  usb_find_devices()
  usb_get_busses()
  ; ----------------------------------------------------------------------
  
  ; Don't work for me, but don't know why :(
  
  usb_open.q(*dev.usb_device)      ;returns *usb_dev_handle
  usb_close(*dev.usb_dev_handle) ;returns 0 if success
  usb_set_configuration(*dev.usb_dev_handle, configuration.i) ;returns 0 If success
  usb_set_altinterface(*dev.usb_dev_handle, alternate.i)
  usb_resetep(*dev.usb_dev_handle, ep.i) ; usb_resetep is deprecated. Use usb_clear_halt
  usb_clear_halt(*dev.usb_dev_handle, ep.i)
  usb_reset(*dev.usb_dev_handle)
  usb_claim_interface(*dev.usb_dev_handle, hInterface.i)
  usb_release_interface(*dev.usb_dev_handle, hInterface.i)
  
  usb_control_msg(*dev.usb_dev_handle, requesttype.i, request.i, value.i, index.i, *bytes, size.i, timeout.i)
  usb_get_string(*dev.usb_dev_handle, index.i, langid.i, *buf, buflen.i)
  usb_get_string_simple(*dev.usb_dev_handle, index.i, *buf, buflen.i)
  usb_get_descriptor(*dev.usb_dev_handle, type.a, index.a, *buf, size.i)
  usb_get_descriptor_by_endpoint(*dev.usb_dev_handle, ep.i, type.a, index.a, *buf, size.i)
  
  usb_bulk_write(*dev.usb_dev_handle, ep.i, *bytes, size.i, timeout.i)
  usb_bulk_read(*dev.usb_dev_handle, ep.i, *bytes, size.i, timeout.i)
  
  usb_interrupt_write(*dev.usb_dev_handle, ep.i, *bytes, size.i, timeout.i)
  usb_interrupt_read(*dev.usb_dev_handle, ep.i, *bytes, size.i, timeout.i)
  
;   usb_get_driver_np(*dev.usb_dev_handle, hInterface.i, *name, namelen.i)
;   usb_detach_kernel_driver_np(*dev.usb_dev_handle, hInterface.i)
  
EndImport


; http://macappstore.org/libusb-compat/

; Press Command+Space And type Terminal And press enter/Return key.
; Run in Terminal app:
; ruby -e "$(curl -fsSL https://raw.githubusercontent.com/Homebrew/install/master/install)" < /dev/null 2> /dev/null
; And press enter/Return key. Wait For the command To finish.
; Run:
; brew install libusb-compat
; IDE Options = PureBasic 5.41 LTS (Windows - x86)
; CursorPosition = 337
; FirstLine = 302
; Folding = -
; EnableXP